-- #############################################################################
-- # Calculated Capacity B-3.0 - Lua application for JETI DC/DS transmitters
-- #
-- # Copyright (c) 2019, Walter Loetscher
-- # All rights reserved.
-- #
-- # License: Share alike
-- # Can be used and changed non commercial
-- #
-- # B-3.0 Simple Version for DS/DC16
-- #############################################################################
--------------------------------------------------------------------------------
-- Locals
local version = "B-3.0"
local sThrustControl, sCapacity, sAltCapacity, sThrust100, sThrust75, sThrust50, sThrust25, sThrust00,
      sKnockOff, sBingo, sSafetySwitch, sResetSwitch, sAccuSwitch, sReport, actualFuel,
			vrs, vss, vgs, acs, idle
local i, diff, cRate 
local t = {}                             -- t = Throttle 0-100 %
local start, change, oneSecond = true, true, false
local alarm50, alarm25, alarm20, alarm15, alarm10, alarmknockoff, alarmbingo = true, true, true, true, true, true, true
local sMultFlight, sIntCap, actualTime = 2, 100, 0
local throttle, thrust, addThrottle, avgThrottle, count = 0, 0, 0, 0, 1
local dispFuel, cellDec, maxFuel, coe = 100, 20, 100000, 1                                                       
local sAltEn, capacity, delay, mem, debugmem = 0, 0, 0, 0, 0
--------------------------------------------------------------------------------
-- Telemetry and Data Verification
local function printTelem()
	if throttle and (sCapacity > 0) and (sThrust100 > -1) then
	lcd.drawText(10,0,string.format("%s%%",dispFuel),FONT_MAXI)
	lcd.drawNumber(5,34,capacity,FONT_NORMAL)
	lcd.drawText(45,34,"mAh")	
	lcd.drawNumber(15,50,thrust,FONT_NORMAL)
	lcd.drawText(45,50,"%  Throttle")
	if cRate then
		lcd.drawText(80,34,string.format("%02dC",cRate, "C"),FONT_NORMAL)
		end
	else
	lcd.drawText(10,10,"Missing Datas",FONT_NORMAL)
	end
end
--------------------------------------------------------------------------------
-- Save
local function tcChanged(value)
	sThrustControl=value
	system.pSave("sThrustControl",value)
	change = true
end
local function caChanged(value)
	sCapacity=value
	system.pSave("sCapacity",value)
	change = true
end
local function acChanged(value)
	sAltCapacity=value
	system.pSave("sAltCapacity",value)
	change = true
end
local function T100Changed(value)
	sThrust100=value
	system.pSave("sThrust100",value)
	change = true
end
local function T75Changed(value)
	sThrust75=value
	system.pSave("sThrust75",value)
	change = true
end
local function T50Changed(value)
	sThrust50=value
	system.pSave("sThrust50",value)
	change = true
end
local function T25Changed(value)
	sThrust25=value
	system.pSave("sThrust25",value)
	change = true
end
local function T00Changed(value)
	sThrust00=value
	system.pSave("sThrust00",value)
	change = true
end
local function koChanged(value)
	sKnockOff=value
	system.pSave("sKnockOff",value)
end
local function biChanged(value)
	sBingo=value
	system.pSave("sBingo",value)
end
local function swChanged(value) -- 1=Safe, -1=Disarmed, 0=not set
	sSafetySwitch=value
	system.pSave("sSafetySwitch",value)
	change = true
end
local function rsChanged(value) -- 0=Reset, -1=off
	sResetSwitch=value
	system.pSave("sResetSwitch",value)
	change = true
end
local function rfChanged(value)
	sReport=value
	system.pSave("sReport",value)
	change = true
end
local function mfChanged(value) -- 0=Reset, -1=off
	sMultFlight=value
	system.pSave("sMultFlight",value)
	change = true
end
local function icChanged(value)
	sIntCap=value
	system.pSave("sIntCap",value)
end
local function asChanged(value) -- 1=Safe, -1=Disarmed, 0=not set
	sAccuSwitch=value
	system.pSave("sAccuSwitch",value)
	change = true
end
--------------------------------------------------------------------------------
-- Functions
local function resetAlarm()
	alarm50 = true
	alarm25 = true
	alarm20 = true
	alarm15 = true
	alarm10 = true
	alarmknockoff = true
	alarmbingo = true
end
local function getValues()
	for i = 1, 100 do t[i] = 0 end
	if sAltEn == 1 and sAltCapacity > 0 then capacity = sAltCapacity
	else capacity = sCapacity
	end	
	t[0] = math.floor(sThrust00 * 10000 / capacity / 3.6)/10		-- Dezimalstellen korrigiert
	t[25] = math.floor(sThrust25 * 10000 / capacity / 3.6)
	t[50] = math.floor(sThrust50 * 10000 / capacity / 3.6)
	t[75] = math.floor(sThrust75 * 10000 / capacity / 3.6)
	t[100] = math.floor(sThrust100 * 10000 / capacity / 3.6)
end
local function initDatas()
	if sMultFlight == 1 then actualFuel = sIntCap * 1000	
	else
	actualFuel = maxFuel	
	end
	actualTime = system.getTimeCounter()
	start = false
end
local function checkSwitches()
	throttle = system.getInputsVal(sThrustControl)
	vss = system.getInputsVal(sSafetySwitch)
	vrs = system.getInputsVal(sResetSwitch)
	vgs = system.getInputsVal(sReport)
	acs = system.getInputsVal(sAccuSwitch)
		if vrs == 1 then 
		actualFuel = maxFuel 
		resetAlarm() 
		end
		if vgs == 1 then 
		delay = delay + 1	-- Fehler?
			if delay == 1 then system.playNumber(dispFuel, 0, "%") end
			if delay == 100 then delay = 0 end
		else
		delay = 0
		end
		if acs == 1 and sAltEn == 0 then sAltEn = 2 end
		if acs == -1 and sAltEn == 2 then sAltEn = 1 change = true system.pSave("sAltEn", sAltEn) end
		if acs == 1 and sAltEn == 1 then sAltEn = 3 end
		if acs == -1 and sAltEn == 3 then sAltEn = 0 change = true system.pSave("sAltEn", sAltEn) end
end
local function curve()
	if t[75] < 0 then
		if t[50] > -1 then t[75] = t[50]+((t[100]-t[50])//2)
			elseif t[25] > -1 then
				t[50] = ((t[100]-t[25])//3) + t[25]
				t[75] = t[100]-((t[100]-t[25])//3)
			else t[75] = t[100] - ((t[100] - t[0])//4)		
		end
	end
	if t[50] < 0 then
		if t[25] > -1 then t[50] = t[25]+((t[75]-t[25])//2)
			else t[50] = t[75] - ((t[75]-t[0])//3)
		end	
	end
	if t[25] < 0 then t[25] = t[0]+((t[50]-t[0])//2)	end
end
local function fill()
	diff = (t[25] - t[0]) / 25
	for i = 1, 24 do t[i] = t[i-1] + diff end
	diff = (t[50] - t[25]) / 25
	for i = 26, 49 do t[i] = t[i-1] + diff end
	diff = (t[75] - t[50]) / 25
	for i = 51, 74 do t[i] = t[i-1] + diff end
	diff = (t[100] - t[75]) / 25
	for i = 76, 99 do t[i] = t[i-1] + diff end
end
local function checkState()
	if vss == nil then vss = -1 end
end
local function alarm()
	if (sBingo ~= 0) or (sKnockOff ~= 0) then
		if dispFuel <= 50 and alarm50 then
		system.playNumber(50, 0, "%")
		alarm50 = false
		end
		if dispFuel <= (sKnockOff) and alarmknockoff then
		system.playFile("CC_Knockoff.wav", AUDIO_BACKGROUND)
		alarmknockoff = false
		end
		if dispFuel <= (sBingo) and alarmbingo then
		system.playFile("CC_Bingo.wav", AUDIO_BACKGROUND)
		alarmbingo = false
		end
	else
	if dispFuel <= 50 and alarm50 then
		system.playNumber(50, 0, "%")
		alarm50 = false
	end
	if dispFuel <= 25 and alarm25 then
		system.playNumber(25, 0, "%")
		alarm25 = false
	end
	if dispFuel <= 20 and alarm20 then
		system.playNumber(20, 0, "%")
		alarm20 = false
	end
	if dispFuel <= 15 and alarm15 then
		system.playNumber(15, 0, "%")
		alarm15 = false
	end
	if dispFuel <= 10 and alarm10 then
		system.playNumber(10, 0, "%")
		alarm10 = false
	end
	end
end
local function info()	
	cRate = math.floor(sThrust100 / capacity * 100)
end
--------------------------------------------------------------------------------
-- Debug
local function debug()	
print("mem=", mem)
print("vss=", vss)
end
--------------------------------------------------------------------------------
-- Loop
local function loop()
	if change == true then 
	getValues() curve() fill() 
	idle = t[0]
	change = false 
	end
	if start then initDatas() end
	checkSwitches()	
	if throttle then
		checkState()
		thrust = math.floor((throttle * 100 + 100) / 2)
		if system.getTimeCounter() >= (actualTime + 1000) 
		then
		oneSecond = true
		end
		if oneSecond and (count > 0) then
			oneSecond = false
			actualTime = (actualTime + 1000)					
			avgThrottle = math.floor(addThrottle / count)
			coe = (100 - cellDec + cellDec * actualFuel / maxFuel) / 100
			if vss == -1  then actualFuel = actualFuel - coe * t[avgThrottle]
			elseif vss == 1 then actualFuel = actualFuel - coe * idle
			end
			dispFuel = (actualFuel // 100) / 10
			info()
			alarm()
			count = 0
			addThrottle = 0
			if sMultFlight == 1 then icChanged(dispFuel) end
		else
			addThrottle = addThrottle + thrust
			count = count + 1
		end
	end
	debugmem = math.modf(collectgarbage('count'))
	if mem < debugmem then mem = debugmem end
	debug()
end
--------------------------------------------------------------------------------
-- Form Initialization
local function initForm(subform)
	local mfOptions = {"yes", "no"}
	form.addLabel({label="--> Mandatory Entrys <--", font=FONT_MINI})
	form.addRow(2)
	form.addLabel({label="Throttle Control", font=FONT_BOLD})
	form.addInputbox(sThrustControl,true,tcChanged, {font=FONT_BOLD})
	form.addLabel({label="  --> Set Proportional!", font=FONT_MINI})
	form.addRow(2)
	form.addLabel({label="Capacity mAh", font=FONT_BOLD})
	form.addIntbox(sCapacity,25,10000,0,0,25,caChanged, {font=FONT_BOLD})
	form.addRow(2)
	form.addLabel({label="Throttle 100%", font=FONT_BOLD})
	form.addIntbox(sThrust100,-1,10000,0,1,1,T100Changed, {font=FONT_BOLD})
	form.addLabel({label="  --> Ampère ", font=FONT_MINI})
	form.addLabel({label="___________________________________________________________"})
	form.addLabel({label="  --> Optional Entrys <--", font=FONT_MINI})
	form.addRow(2)
	form.addLabel({label="Throttle 75%", font=FONT_BOLD})
	form.addIntbox(sThrust75,-1,10000,0,1,1,T75Changed, {font=FONT_BOLD})
	form.addLabel({label="  --> -0.1 for Automatic Interpolation", font=FONT_MINI})
	form.addRow(2)
	form.addLabel({label="Throttle 50%", font=FONT_BOLD})
	form.addIntbox(sThrust50,-1,10000,0,1,1,T50Changed, {font=FONT_BOLD})
	form.addRow(2)
	form.addLabel({label="Throttle 25%", font=FONT_BOLD})
	form.addIntbox(sThrust25,-1,10000,0,1,1,T25Changed, {font=FONT_BOLD})
	form.addRow(2)
	form.addLabel({label="Throttle 0%", font=FONT_BOLD})
	form.addIntbox(sThrust00,0,32767,0,2,1,T00Changed, {font=FONT_BOLD})
	form.addRow(2)
	form.addLabel({label="Prewarning %", font=FONT_BOLD})
	form.addIntbox(sKnockOff,0,100,0,0,1,koChanged, {font=FONT_BOLD})
	form.addRow(2)
	form.addLabel({label="Warning (Bingo) %", font=FONT_BOLD})
	form.addIntbox(sBingo,0,100,0,0,1,biChanged, {font=FONT_BOLD})
	form.addRow(2)
	form.addLabel({label="Safety Switch", font=FONT_BOLD})
	form.addInputbox(sSafetySwitch,true,swChanged, {font=FONT_BOLD})
	form.addRow(2)
	form.addLabel({label="Reset Switch", font=FONT_BOLD})
	form.addInputbox(sResetSwitch,true,rsChanged, {font=FONT_BOLD})
	form.addLabel({label="  --> Toggle Switch recommended", font=FONT_MINI})
	form.addRow(2)
	form.addLabel({label="Report Actual Fuel", font=FONT_BOLD})
	form.addInputbox(sReport,true,rfChanged, {font=FONT_BOLD})
	form.addRow(2)
	form.addLabel({label="Multiple Flights?", font=FONT_BOLD})
	form.addSelectbox(mfOptions, sMultFlight, 2, mfChanged, {font=FONT_BOLD})
	form.addRow(2)
	form.addLabel({label="Alternate Accu", font=FONT_BOLD})
	form.addIntbox(sAltCapacity,0,10000,0,0,25,acChanged, {font=FONT_BOLD})
	form.addRow(2)
	form.addLabel({label="Toggle Alt Accu", font=FONT_BOLD})
	form.addInputbox(sAccuSwitch,true,asChanged, {font=FONT_BOLD})
	form.addLabel({label="Toggle Switch", font=FONT_MINI})
end
--------------------------------------------------------------------------------
-- Init
local function init()
	system.registerForm(1,MENU_MAIN,"Calculated Capacity B 3.0",initForm , nil,printForm)
	system.registerTelemetry(1,"Calculated Capacity B 3.0",2,printTelem)
	-- Saved Items
	sThrustControl = system.pLoad("sThrustControl",nil)	-- Throttle (P4 for Mode 2), use Proportional!
	sCapacity = system.pLoad("sCapacity",2000)			-- Capacity mAh or mL
	sThrust100 = system.pLoad("sThrust100",150)			-- Current Throttle 100%
	sThrust75 = system.pLoad("sThrust75",-1)			-- Current Throttle 75%
	sThrust50 = system.pLoad("sThrust50",-1)			-- Current Throttle 50%
	sThrust25 = system.pLoad("sThrust25",-1)			-- Current Throttle 25%
	sThrust00 = system.pLoad("sThrust00",0)				-- Current Throttle 0%
	sKnockOff = system.pLoad("sKnockOff",30)         	-- Prewarning
	sBingo = system.pLoad("sBingo",25)               	-- Final Warning
	sSafetySwitch = system.pLoad("sSafetySwitch")		-- Engine Safety Switch
	sResetSwitch = system.pLoad("sResetSwitch")			-- Reset capacity to 100%
	sMultFlight = system.pLoad("sMultFlight",2)			-- Value Stored for next Flight	
	sReport = system.pLoad("sReport")               	-- Report actual Capacity
	sIntCap = system.pLoad("sIntCap",100)				-- Store Value for multiple Flights
	sAltCapacity = system.pLoad("sAltCapacity",0)		-- Capacity mAh or mL
	sAccuSwitch = system.pLoad("sAccuSwitch")			-- Switch to Alternate Accu 	
	sAltEn = system.pLoad("sAltEn",0)					-- 0 = primary Accu, 1 = alternate Accu
end
--------------------------------------------------------------------------------
return {init=init, loop=loop, author="Walter Loetscher", version=version, name="Calculated Capacity B 3.0"}
--------------------------------------------------------------------------------
